'use strict';
var jsesc = require('jsesc');
var minitable = require('minitable');
var assertVar = require('../xm/assertVar');
var typeOf = require('../xm/typeOf');
var StyledOut = require('../xm/lib/StyledOut');
var sorter = require('./sorter');
var Formatter = (function () {
    function Formatter(expose, output) {
        if (output === void 0) { output = null; }
        assertVar(output, StyledOut, 'output', true);
        this.expose = expose;
        this.output = (output || new StyledOut());
    }
    Formatter.prototype.printCommands = function () {
        var _this = this;
        var builder = minitable.getBuilder(this.output.getWrite(), this.output.getStyle());
        assertVar(builder, 'object', 'builder');
        var headers = builder.createType('headers', [
            { name: 'title' },
        ]);
        var divider = builder.createType('divider', [
            { name: 'main' },
        ]);
        var commands = builder.createType('commands', [
            { name: 'command' },
            { name: 'short' },
            { name: 'label' }
        ], {
            inner: '   ',
            rowSpace: 0
        });
        var lines = builder.createType('line', [
            { name: 'label' }
        ], {
            rowSpace: 0
        });
        var examples = builder.createType('examples', [
            { name: 'command' },
            { name: 'label' }
        ], {
            inner: '   '
        });
        headers.init();
        divider.init();
        commands.init();
        lines.init();
        examples.init();
        var commandOptNames = [];
        var globalOptNames = [];
        var allCommands = this.expose.commands.keys();
        var allGroups = this.expose.groups.values();
        var sortOptionName = function (one, two) {
            return sorter.sortOption(_this.expose.options.get(one), _this.expose.options.get(two));
        };
        var optKeys = this.expose.options.keys().sort(sortOptionName);
        var firstHeader = true;
        var addHeader = function (title) {
            if (!firstHeader) {
                addDivider();
            }
            builder.closeAll();
            firstHeader = false;
            headers.next();
            headers.row.title.out.accent('>> ').plain(title).accent(':').line();
            addDivider();
        };
        var addDivider = function () {
            builder.closeAll();
            divider.next();
            divider.row.main.out.line('   ');
        };
        var addOption = function (name) {
            commands.next();
            var option = _this.expose.options.get(name);
            var command = commands.row.command.out;
            var label = commands.row.label.out;
            if (!option) {
                command.indent(1).sp().accent('--').plain(name).ln();
                label.indent(1).warning('<undefined>').ln();
            }
            else if (!option.hidden) {
                command.indent(1).sp().accent('--').plain(name);
                if (option.placeholder) {
                    command.sp().muted('<').plain(option.placeholder).muted('>');
                }
                command.ln();
                if (option.short) {
                    commands.row.short.out.accent('-').line(option.short);
                }
                label.accent(' > ').plain(option.description);
                label.sp().accent('(').plain(option.type);
                label.plain((option.default ? ', default: ' + option.default : ''));
                label.accent(')').ln();
                if (option.enum.length > 0) {
                    label.indent().accent(' [ ').plain(option.enum.map(function (value) {
                        if (typeOf.isNumber(value)) {
                            return value;
                        }
                        var str = ('' + value);
                        if (/^[\w_-]*$/.test(str)) {
                            return str;
                        }
                        return '\'' + jsesc(('' + value), {
                            quotes: 'single'
                        }) + '\'';
                    }).join(', ')).accent(' ]').ln();
                }
                addNote(option.note);
            }
        };
        var addCommand = function (cmd, group) {
            if (cmd.hidden) {
                return;
            }
            commands.next();
            var command = commands.row.command.out;
            command.indent(1).plain(cmd.name);
            if (cmd.variadic.length > 0) {
                command.sp().muted('<').plain(cmd.variadic.join(', ')).muted('>');
            }
            command.ln();
            commands.row.label.out.line(cmd.label);
            addNote(cmd.note);
            addExamples(cmd.examples);
            cmd.options.filter(function (name) {
                return (commandOptNames.indexOf(name) < 0) && (globalOptNames.indexOf(name) < 0);
            }).sort(sortOptionName).forEach(function (name) {
                addOption(name);
            });
        };
        var addNote = function (note) {
            if (note && note.length > 0) {
                note.forEach(function (note) {
                    commands.row.label.out.indent().accent(' : ').line(String(note));
                });
            }
        };
        var addExamples = function (list) {
            if (list && list.length > 0) {
                builder.closeAll();
                lines.next();
                lines.row.label.out.line();
                lines.close();
                list.forEach(function (cols) {
                    if (cols.length === 1) {
                        lines.next();
                        lines.row.label.out.indent().line(String(cols[0]));
                    }
                    else if (cols.length > 1) {
                        examples.next();
                        examples.row.command.out.indent(2).accent(' $ ').line(String(cols[0]));
                        examples.row.label.out.accent(' : ').line(String(cols[1]));
                    }
                });
                examples.close();
                lines.next();
                lines.row.label.out.line();
            }
        };
        optKeys.forEach(function (name) {
            var option = _this.expose.options.get(name);
            if (option.command && !option.hidden) {
                commandOptNames.push(option.name);
            }
        });
        optKeys.forEach(function (name) {
            var option = _this.expose.options.get(name);
            if (option.global && !option.command && !option.hidden) {
                globalOptNames.push(option.name);
            }
        });
        if (allGroups.length > 0) {
            this.expose.groups.values().sort(sorter.sortGroup).forEach(function (group) {
                var contents = _this.expose.commands.values().filter(function (cmd) {
                    return cmd.groups.indexOf(group.name) > -1;
                });
                if (contents.length > 0) {
                    addHeader(group.label);
                    contents.sort(group.sorter).forEach(function (cmd) {
                        addCommand(cmd, group);
                        var i = allCommands.indexOf(cmd.name);
                        if (i > -1) {
                            allCommands.splice(i, 1);
                        }
                    });
                    if (group.options.length > 0) {
                        group.options.filter(function (name) {
                            return (commandOptNames.indexOf(name) < 0) && (globalOptNames.indexOf(name) < 0);
                        }).sort(sortOptionName).forEach(function (name) {
                            addOption(name);
                        });
                    }
                }
            });
        }
        if (allCommands.length > 0) {
            addHeader('other commands');
            allCommands.forEach(function (name) {
                addCommand(_this.expose.commands.get(name), _this.expose.mainGroup);
            });
        }
        if (commandOptNames.length > 0 && globalOptNames.length > 0) {
            addHeader('global options');
            if (commandOptNames.length > 0) {
                commandOptNames.forEach(function (name) {
                    addOption(name);
                });
            }
            if (globalOptNames.length > 0) {
                globalOptNames.forEach(function (name) {
                    addOption(name);
                });
            }
        }
        builder.flush();
    };
    return Formatter;
})();
module.exports = Formatter;
